local function normalizeTSHighlight() --- {{{
  -- helper {{{
  local hl = function(group, opts)
    if (opts.default == nil) then
      opts.default = true
    end
    vim.api.nvim_set_hl(0, group, opts)
  end
  -- }}}
  -- Misc {{{
  hl('@comment', { link = 'Comment' })
  hl('@none', { bg = 'NONE', fg = 'NONE' })
  hl('@preproc', { link = 'PreProc' })
  hl('@define', { link = 'Define' })
  hl('@operator', { link = 'Operator' })
  -- }}}
  -- Punctuation {{{
  hl('@punctuation.delimiter', { link = 'Delimiter' })
  hl('@punctuation.bracket', { link = 'Delimiter' })
  hl('@punctuation.special', { link = 'Delimiter' })
  -- }}}
  -- Literals {{{
  hl('@string', { link = 'String' })
  hl('@string.regex', { link = 'String' })
  hl('@string.escape', { link = 'SpecialChar' })
  hl('@string.special', { link = 'SpecialChar' })

  hl('@character', { link = 'Character' })
  hl('@character.special', { link = 'SpecialChar' })

  hl('@boolean', { link = 'Boolean' })
  hl('@number', { link = 'Number' })
  hl('@float', { link = 'Float' })
  -- }}}
  -- Functions {{{
  hl('@function', { link = 'Function' })
  hl('@function.call', { link = 'Function' })
  hl('@function.builtin', { link = 'Special' })
  hl('@function.macro', { link = 'Macro' })

  hl('@method', { link = 'Function' })
  hl('@method.call', { link = 'Function' })

  hl('@constructor', { link = 'Special' })
  hl('@parameter', { link = 'Identifier' })
  -- }}}
  -- Keywords {{{
  hl('@keyword', { link = 'Keyword' })
  hl('@keyword.function', { link = 'Keyword' })
  hl('@keyword.operator', { link = 'Keyword' })
  hl('@keyword.return', { link = 'Keyword' })

  hl('@conditional', { link = 'Conditional' })
  hl('@repeat', { link = 'Repeat' })
  hl('@debug', { link = 'Debug' })
  hl('@label', { link = 'Label' })
  hl('@include', { link = 'Include' })
  hl('@exception', { link = 'Exception' })
  -- }}}
  -- Types {{{
  hl('@type', { link = 'Type' })
  hl('@type.builtin', { link = 'Type' })
  hl('@type.qualifier', { link = 'Type' })
  hl('@type.definition', { link = 'Typedef' })

  hl('@storageclass', { link = 'StorageClass' })
  hl('@attribute', { link = 'PreProc' })
  hl('@field', { link = 'Identifier' })
  hl('@property', { link = 'Identifier' })
  -- }}}
  -- Identifiers {{{
  hl('@variable', { link = 'Identifier' })
  hl('@variable.builtin', { link = 'Identifier' })

  hl('@constant', { link = 'Constant' })
  hl('@constant.builtin', { link = 'Special' })
  hl('@constant.macro', { link = 'Define' })

  hl('@namespace', { link = 'Include' })
  hl('@symbol', { link = 'Identifier' })
  -- }}}
  -- Text {{{
  hl('@text', {})
  hl('@text.strong', { bold = true })
  hl('@text.emphasis', { italic = true })
  hl('@text.underline', { underline = true })
  hl('@text.strike', { strikethrough = true })
  hl('@text.title', { link = 'Title' })
  hl('@text.literal', { link = 'String' })
  hl('@text.uri', { link = 'Underlined' })
  hl('@text.math', { link = 'Special' })
  hl('@text.environment', { link = 'Macro' })
  hl('@text.environment.name', { link = 'Type' })
  hl('@text.reference', { link = 'Constant' })

  hl('@text.todo', { link = 'Todo' })
  hl('@text.note', { link = 'SpecialComment' })
  hl('@text.warning', { link = 'WarningMsg' })
  hl('@text.danger', { link = 'ErrorMsg' })
  -- }}}
  -- Tags {{{
  hl('@tag', { link = 'Tag' })
  hl('@tag.attribute', { link = 'Identifier' })
  hl('@tag.delimiter', { link = 'Delimiter' })
  -- }}}
end -- }}}

local function normalizeTerminal() -- {{{
  vim.g.terminal_color_0 = 0
  vim.g.terminal_color_1 = 1
  vim.g.terminal_color_2 = 2
  vim.g.terminal_color_3 = 3
  vim.g.terminal_color_4 = 4
  vim.g.terminal_color_5 = 5
  vim.g.terminal_color_6 = 6
  vim.g.terminal_color_7 = 7
  vim.g.terminal_color_8 = 8
  vim.g.terminal_color_9 = 9
  vim.g.terminal_color_10 = 10
  vim.g.terminal_color_11 = 11
  vim.g.terminal_color_12 = 12
  vim.g.terminal_color_13 = 13
  vim.g.terminal_color_14 = 14
  vim.g.terminal_color_15 = 15
end -- }}}

local function customLinks()
  local cterm = vim.api.nvim_get_hl_by_name('Normal', false)
  local gui = vim.api.nvim_get_hl_by_name('Normal', true)
  vim.api.nvim_set_hl(0, 'Function', { ctermfg = cterm.foreground, fg = gui.foreground })
  vim.api.nvim_set_hl(0, 'Operator', { ctermfg = cterm.foreground, fg = gui.foreground })
end

local augr = vim.api.nvim_create_augroup('config_colorscheme', {});
vim.api.nvim_create_autocmd('Colorscheme', {
  group = augr,
  pattern = '*',
  callback = function()
    normalizeTSHighlight()
    customLinks()
    normalizeTerminal()
  end
})

vim.api.nvim_create_autocmd('Colorscheme', {
  group = augr,
  pattern = 'gruvbox',
  callback = function()
    vim.api.nvim_set_hl(0, 'Directory', { link = 'GruvboxGreen' })
    vim.api.nvim_set_hl(0, 'TelescopeSelection', { link = 'GruvboxOrangeSign' })
    vim.api.nvim_set_hl(0, 'TelescopeMatching', { link = 'GruvboxAqua' })

    vim.api.nvim_set_hl(0, 'NeoTreeGitAdded', { link = 'GruvboxGreen' })
    vim.api.nvim_set_hl(0, 'NeoTreeGitDeleted', { link = 'GruvboxRed' })
    vim.api.nvim_set_hl(0, 'NeoTreeGitModified', { link = 'GruvboxOrange' })
  end
})

vim.api.nvim_create_autocmd('ColorschemePre', {
  group = augr,
  pattern = '*',
  callback = function()
    vim.fn.sign_define('DiagnosticSignError', { text = ' ', texthl = 'DiagnosticSignError' })
    vim.fn.sign_define('DiagnosticSignWarn', { text = ' ', texthl = 'DiagnosticSignWarn' })
    vim.fn.sign_define('DiagnosticSignInfo', { text = ' ', texthl = 'DiagnosticSignInfo' })
    vim.fn.sign_define('DiagnosticSignHint', { text = '', texthl = 'DiagnosticSignHint' })
  end
})

local term = vim.env.TERM
if (term ~= 'linux' or vim.g.neoray)
then
  vim.o.termguicolors = true
  vim.g.gruvbox_contrast_dark = 'hard'
  vim.cmd('colorscheme gruvbox')
end
